package indep_set.LaTeX;

import java.io.PrintWriter;

public class LaTeXTablePrinter {
	
	
	public static String BEGIN_TABULAR = "\\begin{tabular}";
	public static String END_TABULAR = "\\end{tabular}";
	public static String NEW_ROW = "\\\\";
	public static String NEW_PAGE = "\\newpage";
	public static String HLINE = "\\hline";
	public static String TAB = "\t";
	
	private String [] header;
	
	int numCols;
	
	PrintWriter writer;
	boolean headerWritten = false;
	boolean tableDone = false;
	
	public LaTeXTablePrinter(PrintWriter writer){
		this.writer = writer;
	}
	
	public void printHeader(String... header){
		assert(!headerWritten);
		assert(!tableDone);
		this.header = header;
		
		writer.print(BEGIN_TABULAR);
		numCols = header.length;
		writer.println(tabularCols(numCols));
		writer.println(TAB + " " + HLINE);
		writer.print(TAB);
		writer.println(rowToLatex(header));
		writer.println(TAB + " " + HLINE + HLINE );
		headerWritten = true;
	}
	
	public void startNewPage(){
		assert(headerWritten);
		assert(!tableDone);
		writer.println(END_TABULAR);
		writer.println(NEW_PAGE);
		writer.print(BEGIN_TABULAR);
		writer.println(tabularCols(numCols));
		writer.println(TAB + " " + HLINE);
		writer.print(TAB);
		writer.println(rowToLatex(header));
		writer.println(TAB + " " + HLINE + HLINE );	
	}
	
	public void printRow(String... row){
		assert(headerWritten);
		assert(!tableDone);
		
		assert(row.length == numCols);
		writer.print(TAB);
		writer.print(rowToLatex(row));
		writer.println(TAB + HLINE );
	}
	
	public void finish(){
		assert(headerWritten);
		assert(!tableDone);
		
		tableDone = true;
		
		writer.println(END_TABULAR);
	}	
	
//	private String rowToLatex(List<String> row){
//		StringBuilder sb = new StringBuilder();
//		for (int i = 0; i < row.size(); i++){
//			sb.append(row.get(i));
//			if (i < row.size() - 1){
//				sb.append(" & ");
//			}
//		}
//		sb.append(" " + NEW_ROW);
//		return sb.toString();
//	}
	
	private String rowToLatex(String[] row){
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < row.length; i++){
			sb.append(row[i]);
			if (i < row.length - 1){
				sb.append(" & ");
			}
		}
		sb.append(" " + NEW_ROW);
		return sb.toString();
	}
	
	private String tabularCols(int n){ //print {|c|c|c|c|} with n c's
		StringBuilder sb = new StringBuilder();
		sb.append("{|");
		for (int i = 0; i < n; i++){
			sb.append("c|");
		}
		sb.append("}");
		return sb.toString();
	}

}
