package indep_set.LaTeX;

import indep_set.root.Root;

public class TikZRoot extends TikZ {
	
	Root root;
	
	double lvl1length, lvl2length;
	double lvl1step, lvl2step;
	
	double lvl0x;
	double[] lvl1x;
	double[] lvl2x;
	double lvl0y;
	double lvl1y;
	double lvl2y;
	
	
	String rootName = "0";
	String[][] nodeName;
	
	public TikZRoot(Root root){
		super();
		this.root = root;
		calculateLengthAndStep();
		calculateCoordinates();
		genNames();
		
		commentRootCode();
		drawBoundingBox();
		createNodes();
		drawEdges();
		finish();
	}
	
	private void commentRootCode(){
		addCommentLine("Code: " + root.code);
	}
	
	private void calculateLengthAndStep(){
		lvl2step = 0.8;
		lvl2length = lvl2step * (root.lvl2size - 1);
		if (root.lvl1size <= root.lvl2size){
			lvl1step = lvl2step * root.lvl2size / root.lvl1size;
		} else {
			lvl1step = lvl2step;
		}
		lvl1length = lvl1step * (root.lvl1size - 1);
		
	}
	
	private void calculateCoordinates(){
		lvl0y = 2;
		lvl1y = 1;
		lvl2y = 0;
		
		lvl0x = 0;
		lvl1x = new double[root.lvl1size];
		lvl2x = new double[root.lvl2size];
		for (int i = 0; i < root.lvl1size; i++){
			lvl1x[i] = - lvl1length / 2 + lvl1step * i;
		}
		for (int i = 0; i < root.lvl2size; i++){
			lvl2x[i] = - lvl2length / 2 + lvl2step * i;
		}
	}
	
	private void genNames(){
		nodeName = new String[2][];
		nodeName[0] = new String[root.lvl1size];
		nodeName[1] = new String[root.lvl2size];
		for (int i = 0; i < root.lvl1size; i++){
			nodeName[0][i] = "1" + i;
		}
		for (int i = 0; i < root.lvl2size; i++){
			nodeName[1][i] = "2" + i;
		}
	}
	
	private void drawBoundingBox(){
		double x = Math.max(lvl1length, lvl2length)/2;
		addBoundingBox(-x-.5, x+.5, lvl2y - .5, lvl0y + .5);
	}
	
	private void createNodes(){
		addNode(rootName, lvl0x, lvl0y, "");
		for (int i = 0; i < root.lvl1size; i++){
			addNode(nodeName[0][i], lvl1x[i], lvl1y, "");
		}
		for (int i = 0; i < root.lvl2size; i++){
			addNode(nodeName[1][i], lvl2x[i], lvl2y, "");
		}
	}
	
	private void drawEdges(){
		for (int i = 0; i < root.lvl1size; i++){
			addDrawEdge(rootName, nodeName[0][i]);
			for (int j : root.level1.get(i)){
				addDrawEdge(nodeName[0][i], nodeName[1][j]);
			}
		}
	}
}
