package indep_set.compound;

import indep_set.Config;
import indep_set.root.Root;

import java.util.Iterator;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;


/**
 * Iterates through all the compounds with a given root.
 */
public class CompoundIterator implements Iterator<Compound>{
	
	Root root;
	int lvl2size;
	
	boolean hasNext = true;
	
	int minDegree;
		
	int[] lvl2outDegMin;
	int[] lvl2outDegMax;
	int[] lvl2outDegCurrent;
	
	public CompoundIterator(Root root){
		this.root = root;
		this.lvl2size = root.lvl2size;
		computeRange();
		init();
	}
	
	private void computeRange(){
		minDegree = root.lvl1size;
		lvl2outDegMin = new int[lvl2size];
		lvl2outDegMax = new int[lvl2size];
		
		for (int i = 0; i < lvl2size; i++){
			if (Config.root_node_min){
				lvl2outDegMin[i] = root.lvl1size - root.lvl2inDegree(i);
			} else {
				lvl2outDegMin[i] = 0;
			}
			if (Config.root_node_max){
				lvl2outDegMax[i] = root.lvl1size - root.lvl2inDegree(i);
			} else {
				lvl2outDegMax[i] = Config.Delta - root.lvl2inDegree(i);
			}
		}
	}
	
	private void init(){
		lvl2outDegCurrent = new int[lvl2size];
		for (int i = 0; i < lvl2size; i++){
			lvl2outDegCurrent[i] = lvl2outDegMin[i]; 
		}
	}	

	@Override
	public boolean hasNext() {
		return hasNext;
	}

	@Override
	public Compound next() {
		assert(hasNext);		

		Compound toReturn = new Compound(root, lvl2outDegCurrent);
				
		//increase the index for the next iteration
		int i;
		for (i = 0; i < lvl2size; i++){
			if (lvl2outDegCurrent[i] < lvl2outDegMax[i]){
				lvl2outDegCurrent[i]++;
				break;
			} else {
				lvl2outDegCurrent[i] = lvl2outDegMin[i];
			}
		}
		if (i==lvl2size){
			hasNext = false;
		}		
		
		return toReturn;
	}

	@Override
	public void remove() {
		throw new NotImplementedException();		
	}

}
