package indep_set.compound;

import indep_set.Config;
import indep_set.MyUtil;
import indep_set.LaTeX.LaTeXTablePrinter;
import indep_set.LaTeX.TikZRoot;
import indep_set.root.Root;

import java.io.PrintWriter;

public class DataPagePrinter {
	
	Root root;
	LaTeXTablePrinter tablePrinter;
	public PrintWriter writer = null;
	
	boolean isException = false;
	
	int counter = 10;
	
	public DataPagePrinter(Root root, PrintWriter writer){
		this.root = root;
		this.writer = writer;
		this.tablePrinter = new LaTeXTablePrinter(writer);
	}
	
	public void print(){
		if (Config.exceptions_only){
			iterateThroughCompounds();
			if (isException)
				printFooter();
		} else{
			printPageHeader();
			iterateThroughCompounds();
			printFooter();
		}
		
	}
	
	private void iterateThroughCompounds(){		
		CompoundIterator iterator = new CompoundIterator(root);
		Compound compound;
		while(iterator.hasNext()){
			compound = iterator.next();
			compute(compound);
		}
	}
	
	private void printPageHeader(){
		writer.println(new TikZRoot(root).toString());
		printTableHeader();
	}
	
	private void printTableHeader(){
		writer.println();
		tablePrinter.printHeader(
				"Level-2 down-degrees", 
				"$\\Pi(G)$", 
				"$\\Pi(G-x)$", 
				"$\\Pi(G-x-N_x)$",
				"Difference",  //"$\\Pi(G) - \\Pi(G-x) - \\Pi(G-x-N_x)$", 
				"Negative?");
	}
	
	private void printFooter(){
		tablePrinter.finish();
		writer.println();
		if (!Config.exceptions_only)
			writer.println("\\newpage");
		writer.println();
		writer.println("\\bigskip");
		writer.println();
	}
	
	private void compute(Compound compound){
		double P, P1, P2;
		boolean currentNegative, currentNonPositive;
		P = compound.weight();
		P1 = compound.weight1();
		P2 = compound.weight2();
		currentNegative = (P - P1 - P2 < - Calculation.tolerance);
		currentNonPositive = (P - P1 - P2 < Calculation.tolerance);
		
		if (currentNegative){
			System.out.println("NEGATIVE: "
					+ "[" + compound.root.code + "] "
					+ MyUtil.intArrayToString(compound.outDegree));
		}
		
		if (Config.exceptions_only){
			if (currentNonPositive){ //only print table when there is some non-positive, and only print nonpositive rows
				if (isException){ //headers already printed
					addNewTableRow(compound.outDegree, P, P1, P2, P - P1 - P2, currentNegative);
					if (counter++ % 50 == 0)
						tablePrinter.startNewPage();
				} else {
					isException = true;
					printPageHeader();
					addNewTableRow(compound.outDegree, P, P1, P2, P - P1 - P2, currentNegative);
					counter++;
				}
			}
		} else{
			addNewTableRow(compound.outDegree, P, P1, P2, P - P1 - P2, currentNegative);
			if (counter++ % 50 == 0)
				tablePrinter.startNewPage();
		}
	}
	
	private void addNewTableRow(int[] outDegree, double P, double P1, double P2, double diff, boolean negative){
		tablePrinter.printRow(
				MyUtil.LaTeXMath(MyUtil.intArrayToString(outDegree)),
				MyUtil.LaTeXMath(MyUtil.decimalTrim(P)),
				MyUtil.LaTeXMath(MyUtil.decimalTrim(P1)),
				MyUtil.LaTeXMath(MyUtil.decimalTrim(P2)),
				MyUtil.LaTeXMath(MyUtil.decimalTrim(Math.abs(diff)< Calculation.tolerance? 0 : diff)),
				(negative ? "WARNING" : ""));
	}
	
	

}
