package indep_set.gen_root_code;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Given parameters n, k, iterates through all subsets of size k in {1, 2, ..., n}
 */
public class AllSubsets implements Iterable<List<Integer>> {

	int n, k;
	List<Integer> starter;
	
	public AllSubsets(int n, int k){
		this.n = n;
		this.k = k;
	}
	
	public AllSubsets(int n, int k, List<Integer> starter){
		this(n,k);
		this.starter = starter;
		assert(starter.size() == k);
	}
	
	@Override
	public Iterator<List<Integer>> iterator() {
		if (starter == null)
			return new AllSubsetsIterator(n,k);
		else
			return new AllSubsetsIterator(n,k, starter);
	}
	
	
	public class AllSubsetsIterator implements Iterator<List<Integer>>{
		int n, k;
		ArrayList<Integer> subset = new ArrayList<Integer>(k);
		boolean hasNext = true;
		public AllSubsetsIterator(int n, int k){
			this.n = n;
			this.k = k;
			assert(n >= k && k > 0); 
			init();
		}
		
		public AllSubsetsIterator(int n, int k, List<Integer> starter){
			this.n = n;
			this.k = k;
			assert(n >= k && k > 0); 
			subset = new ArrayList<Integer>(starter);
		}
		
		private void init(){
			for (int i = 1; i <= k; i++){
				subset.add(i);
			}
		}
		
		private void moveToNextSubset(){
			if (subset.get(0) == n-k+1){
				hasNext = false;
				return;
			}
			int a;
			for (int i = k-1; i >=0; i--){
				a = subset.get(i);
				if (a == n-k+1+i)
					continue;
				else{
					for (int j = i; j < k; j++){
						subset.set(j, a+1+j-i);
					}
					return;
				}				
			}
			assert(false); //shouldn't reach here
		}
		
		@Override
		public boolean hasNext() {
			return hasNext;
		}
		@Override
		public List<Integer> next() {
			List<Integer> toReturn = new ArrayList<Integer>(subset);
			moveToNextSubset();
			return toReturn;
		}
		@Override
		public void remove() {
			throw new UnsupportedOperationException();
		}		
	}	
}
