package indep_set.run;

import indep_set.Config;
import indep_set.LaTeX.LaTeXTemplate;
import indep_set.LaTeX.TikZRoot;
import indep_set.root.RootCode;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Makes drawing of all roots. 
 * Publishes to 
 * latex\index.tex
 */
public class RunDrawRoot {
	File inFile, outFile;
	File templateFile = new File("latex\\template.tex");
	
	public RunDrawRoot(File inFile, File outFile){
		this.inFile = inFile;
		this.outFile = outFile;
	}
	
	public void run() throws IOException{
		BufferedReader reader = null;
		PrintWriter writer = null;
		BufferedReader templateReader = null;
		LaTeXTemplate template = null;
		
		try {
			reader = new BufferedReader(new FileReader(inFile));
			writer = new PrintWriter(new FileWriter(outFile));
			templateReader = new BufferedReader(new FileReader(templateFile));
			template = new LaTeXTemplate(templateReader);
			
		
			writer.println(template.header());
			
			writer.println(Config.configLaTeX());
			
            String l;
            
            while ((l = reader.readLine()) != null) {
            	if (l.trim().isEmpty()) continue;
            	if (l.startsWith("%")) continue;
                writer.println(new TikZRoot(new RootCode(l).getRoot()));
            }
            
            writer.println(template.footer());
            
        } finally {
            if (reader != null) {
            	reader.close();
            }
            if (writer !=null){
            	writer.close();
            }
        }
	}
	
	public static void runRegular(){
		File inFile = new File(RunGenerateRootCodes.outFileName());
		File outFile = new File("latex\\index.tex");
		try{
			new RunDrawRoot(inFile, outFile).run();
		} catch (Exception e){
			e.printStackTrace();
		}
	}

	public static void main(String[] args) throws IOException {
		runRegular();
	}
	
	
}
